//
//  JULIA v3
//
//  Julia set generator for the TI-89 calculator
//
//  (c) 2002 by Stephan Walter <stephan.walter@gmx.ch>
//  Released under the Beer-ware License
//
//  "THE BEER-WARE LICENSE" (Revision 42):
//  <stephan.walter@gmx.ch> wrote this file.   As long as you retain this
//  notice you can do whatever you want with this stuff.  If we meet some
//  day,  and you think this stuff is worth it,  you can buy me a beer in
//  return.                                                Stephan Walter
//

#define USE_TI89		// Programm luft nur auf einem TI-89

#define OPTIMIZE_ROM_CALLS	// etwas schneller
#define SAVE_SCREEN		// Bildschirminhalt speichern und wiederherstellen
#define NO_EXIT_SUPPORT
#define NO_CALC_DETECT

#include <tigcclib.h>

/*	SCALE = 1 << SCALE_BITS
	kann nicht direkt eingesetzt werden
	Bei grsseren Werten kommt es zu Fehlern  */
#define SCALE_BITS 26
#define SCALE 67108864


void draw_julia_int(float fcim, float fcre, float fzre_start, float fzre_end,
		float fzim_start, float fzim_end, short maxit)
{
	short i,y,x;
	long zre,zim,zre1,zim1,zre_neu; //,zim_neu;

	long cim = fcim * SCALE;
	long cre = fcre * SCALE;
	long zre_start = fzre_start * SCALE;
	long zre_end = fzre_end * SCALE;
	long zim_start = fzim_start * SCALE;
	long zim_end = fzim_end * SCALE;

	long dzre = (zre_end-zre_start) / 160;
	long dzim = (zim_end-zim_start) / 100;

	zim1 = zim_start;

	for(y=0;y<100;y++) {
		zre = zre1 = zre_start;
		for(x=0;x<160;x++) {
			zim = zim1;
			GraySetAMSPlane(LIGHT_PLANE);
			DrawPix(x,y,A_NORMAL);
			GraySetAMSPlane(DARK_PLANE);
			DrawPix(x,y,A_NORMAL);

			for(i=0;i<maxit;i++)
			{	/*	zre := zre^2 - zim^2 = (zre-zim) * (zre+zim)
					zim := 2 * zre * zim	*/
				zre_neu = ((long long)(zre-zim) * (long long)(zre+zim))>>SCALE_BITS;
				zim = (((long long)zre*(long long)zim)>>(SCALE_BITS-1)) + cim;
				zre = zre_neu + cre;
				if((abs(zre)>2*SCALE) && (abs(zim)>2*SCALE)) {
					if(i>=5)
						DrawPix(x,y,A_REVERSE);
					if(i>10 || (i<5 && i>=2)) {
						GraySetAMSPlane(LIGHT_PLANE);
						DrawPix(x,y,A_REVERSE);
					}
					break;
				}
			}
			zre = zre1 += dzre;
		}
		zim1 += dzim;
		if(kbhit()) return;
	}
}


void _main(void)
{
	/*	Achtung Hack: Diese 3 char-Arrays mssen in dieser
		Reihenfolge aufeinanderfolgen! */
	char siter[20]="50";
	char scimag[20]=".7";
	char screal[20]={173,'.','2',0}; /* Spezielles Minuszeichen */
	float cre,cim;
	short maxit;

	HANDLE handle = DialogNewSimple (123, 83);

	DialogAddTitle (handle, "JULIA v3", BT_OK, BT_CANCEL);
	DialogAddText (handle, 20, 14, "(c) 2002 Stephan Walter");
	DialogAddText (handle, 17, 21, "http://cowww.epfl.ch/~swalter/");
	DialogAddRequest (handle, 16, 34, "c real", 0, 20, 10);
	DialogAddRequest (handle, 15, 46, "c imag", 20, 20, 10);
	DialogAddRequest (handle, 6, 58, "max. Iter", 40, 4, 4);

	while (DialogDo (handle, CENTER, CENTER, screal, NULL) == KEY_ENTER) {
		clrscr();

		cre=atof(screal);
		cim=atof(scimag);
		maxit=atoi(siter);

		GrayOn();

		draw_julia_int(cim, cre, -1.6, 1.6, 1, -1, maxit);

		ngetchx();
		GrayOff();
	}
}
